@@ -1,26 +1,12 @@
 #!/usr/bin/python
 #
-# Created on Aug 25, 2016
 # @author: Gaurav Rastogi (grastogi@avinetworks.com)
 #          Eric Anderson (eanderson@avinetworks.com)
 # module_check: supported
 # Avi Version: 17.1.1
 #
-#
-# This file is part of Ansible
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
+# Copyright: (c) 2017 Gaurav Rastogi, <grastogi@avinetworks.com>
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 #
 
 ANSIBLE_METADATA = {'metadata_version': '1.1',
@@ -43,7 +29,19 @@ options:
         description:
             - The state that should be applied on the entity.
         default: present
-        choices: ["absent","present"]
+        choices: ["absent", "present"]
+    avi_api_update_method:
+        description:
+            - Default method for object update is HTTP PUT.
+            - Setting to patch will override that behavior to use HTTP PATCH.
+        version_added: "2.5"
+        default: put
+        choices: ["put", "patch"]
+    avi_api_patch_op:
+        description:
+            - Patch operation to use when using avi_api_update_method as patch.
+        version_added: "2.5"
+        choices: ["add", "replace", "delete"]
     apdex_response_threshold:
         description:
             - If a client receives an http response in less than the satisfactory latency threshold, the request is considered satisfied.
@@ -51,6 +49,7 @@ options:
             - Greater than this number and the client's request is considered frustrated.
             - Allowed values are 1-30000.
             - Default value when not specified in API or module is interpreted by Avi Controller as 500.
+            - Units(MILLISECONDS).
     apdex_response_tolerated_factor:
         description:
             - Client tolerated response latency factor.
@@ -62,6 +61,7 @@ options:
             - Satisfactory client to avi round trip time(rtt).
             - Allowed values are 1-2000.
             - Default value when not specified in API or module is interpreted by Avi Controller as 250.
+            - Units(MILLISECONDS).
     apdex_rtt_tolerated_factor:
         description:
             - Tolerated client to avi round trip time(rtt) factor.
@@ -76,6 +76,7 @@ options:
             - A pageload includes the time for dns lookup, download of all http objects, and page render time.
             - Allowed values are 1-30000.
             - Default value when not specified in API or module is interpreted by Avi Controller as 5000.
+            - Units(MILLISECONDS).
     apdex_rum_tolerated_factor:
         description:
             - Virtual service threshold factor for tolerated page load time (plt) as multiple of apdex_rum_threshold.
@@ -88,6 +89,7 @@ options:
             - Greater than this number and the server response is considered frustrated.
             - Allowed values are 1-30000.
             - Default value when not specified in API or module is interpreted by Avi Controller as 400.
+            - Units(MILLISECONDS).
     apdex_server_response_tolerated_factor:
         description:
             - Server tolerated response latency factor.
@@ -99,6 +101,7 @@ options:
             - Satisfactory client to avi round trip time(rtt).
             - Allowed values are 1-2000.
             - Default value when not specified in API or module is interpreted by Avi Controller as 125.
+            - Units(MILLISECONDS).
     apdex_server_rtt_tolerated_factor:
         description:
             - Tolerated client to avi round trip time(rtt) factor.
@@ -118,41 +121,49 @@ options:
             - A connection between client and avi is considered lossy when more than this percentage of out of order packets are received.
             - Allowed values are 1-100.
             - Default value when not specified in API or module is interpreted by Avi Controller as 50.
+            - Units(PERCENT).
     conn_lossy_timeo_rexmt_threshold:
         description:
             - A connection between client and avi is considered lossy when more than this percentage of packets are retransmitted due to timeout.
             - Allowed values are 1-100.
             - Default value when not specified in API or module is interpreted by Avi Controller as 20.
+            - Units(PERCENT).
     conn_lossy_total_rexmt_threshold:
         description:
             - A connection between client and avi is considered lossy when more than this percentage of packets are retransmitted.
             - Allowed values are 1-100.
             - Default value when not specified in API or module is interpreted by Avi Controller as 50.
+            - Units(PERCENT).
     conn_lossy_zero_win_size_event_threshold:
         description:
             - A client connection is considered lossy when percentage of times a packet could not be trasmitted due to tcp zero window is above this threshold.
             - Allowed values are 0-100.
             - Default value when not specified in API or module is interpreted by Avi Controller as 2.
+            - Units(PERCENT).
     conn_server_lossy_ooo_threshold:
         description:
             - A connection between avi and server is considered lossy when more than this percentage of out of order packets are received.
             - Allowed values are 1-100.
             - Default value when not specified in API or module is interpreted by Avi Controller as 50.
+            - Units(PERCENT).
     conn_server_lossy_timeo_rexmt_threshold:
         description:
             - A connection between avi and server is considered lossy when more than this percentage of packets are retransmitted due to timeout.
             - Allowed values are 1-100.
             - Default value when not specified in API or module is interpreted by Avi Controller as 20.
+            - Units(PERCENT).
     conn_server_lossy_total_rexmt_threshold:
         description:
             - A connection between avi and server is considered lossy when more than this percentage of packets are retransmitted.
             - Allowed values are 1-100.
             - Default value when not specified in API or module is interpreted by Avi Controller as 50.
+            - Units(PERCENT).
     conn_server_lossy_zero_win_size_event_threshold:
         description:
             - A server connection is considered lossy when percentage of times a packet could not be trasmitted due to tcp zero window is above this threshold.
             - Allowed values are 0-100.
             - Default value when not specified in API or module is interpreted by Avi Controller as 2.
+            - Units(PERCENT).
     description:
         description:
             - User defined description for the object.
@@ -160,19 +171,30 @@ options:
         description:
             - Disable node (service engine) level analytics forvs metrics.
             - Default value when not specified in API or module is interpreted by Avi Controller as False.
+        type: bool
     disable_server_analytics:
         description:
             - Disable analytics on backend servers.
             - This may be desired in container environment when there are large number of  ephemeral servers.
             - Default value when not specified in API or module is interpreted by Avi Controller as False.
+        type: bool
     exclude_client_close_before_request_as_error:
         description:
             - Exclude client closed connection before an http request could be completed from being classified as an error.
             - Default value when not specified in API or module is interpreted by Avi Controller as False.
+        type: bool
+    exclude_dns_policy_drop_as_significant:
+        description:
+            - Exclude dns policy drops from the list of errors.
+            - Field introduced in 17.2.2.
+            - Default value when not specified in API or module is interpreted by Avi Controller as False.
+        version_added: "2.5"
+        type: bool
     exclude_gs_down_as_error:
         description:
             - Exclude queries to gslb services that are operationally down from the list of errors.
             - Default value when not specified in API or module is interpreted by Avi Controller as False.
+        type: bool
     exclude_http_error_codes:
         description:
             - List of http status codes to be excluded from being classified as an error.
@@ -181,43 +203,53 @@ options:
         description:
             - Exclude dns queries to domains outside the domains configured in the dns application profile from the list of errors.
             - Default value when not specified in API or module is interpreted by Avi Controller as False.
+        type: bool
     exclude_invalid_dns_query_as_error:
         description:
             - Exclude invalid dns queries from the list of errors.
             - Default value when not specified in API or module is interpreted by Avi Controller as False.
+        type: bool
     exclude_no_dns_record_as_error:
         description:
             - Exclude queries to domains that did not have configured services/records from the list of errors.
             - Default value when not specified in API or module is interpreted by Avi Controller as False.
+        type: bool
     exclude_no_valid_gs_member_as_error:
         description:
             - Exclude queries to gslb services that have no available members from the list of errors.
             - Default value when not specified in API or module is interpreted by Avi Controller as False.
+        type: bool
     exclude_persistence_change_as_error:
         description:
             - Exclude persistence server changed while load balancing' from the list of errors.
             - Default value when not specified in API or module is interpreted by Avi Controller as False.
+        type: bool
     exclude_server_dns_error_as_error:
         description:
             - Exclude server dns error response from the list of errors.
             - Default value when not specified in API or module is interpreted by Avi Controller as False.
+        type: bool
     exclude_server_tcp_reset_as_error:
         description:
             - Exclude server tcp reset from errors.
             - It is common for applications like ms exchange.
             - Default value when not specified in API or module is interpreted by Avi Controller as False.
+        type: bool
     exclude_syn_retransmit_as_error:
         description:
             - Exclude 'server unanswered syns' from the list of errors.
             - Default value when not specified in API or module is interpreted by Avi Controller as False.
+        type: bool
     exclude_tcp_reset_as_error:
         description:
             - Exclude tcp resets by client from the list of potential errors.
             - Default value when not specified in API or module is interpreted by Avi Controller as False.
+        type: bool
     exclude_unsupported_dns_query_as_error:
         description:
             - Exclude unsupported dns queries from the list of errors.
             - Default value when not specified in API or module is interpreted by Avi Controller as False.
+        type: bool
     hs_event_throttle_window:
         description:
             - Time window (in secs) within which only unique health change events should occur.
@@ -369,13 +401,12 @@ extends_documentation_fragment:
     - avi
 '''
 
-
-EXAMPLES = '''
+EXAMPLES = """
   - name: Create a custom Analytics profile object
     avi_analyticsprofile:
-      controller: ''
-      username: ''
-      password: ''
+      controller: '{{ controller }}'
+      username: '{{ username }}'
+      password: '{{ password }}'
       apdex_response_threshold: 500
       apdex_response_tolerated_factor: 4.0
       apdex_rtt_threshold: 250
@@ -429,7 +460,8 @@ EXAMPLES = '''
       hs_security_weak_signature_algo_penalty: 1.0
       name: jason-analytics-profile
       tenant_ref: Demo
-'''
+"""
+
 RETURN = '''
 obj:
     description: AnalyticsProfile (api/analyticsprofile) object
@@ -439,7 +471,7 @@ obj:
 
 from ansible.module_utils.basic import AnsibleModule
 try:
-    from ansible.module_utils.avi import (
+    from ansible.module_utils.network.avi.avi import (
         avi_common_argument_spec, HAS_AVI, avi_ansible_api)
 except ImportError:
     HAS_AVI = False
@@ -449,6 +481,9 @@ def main():
     argument_specs = dict(
         state=dict(default='present',
                    choices=['absent', 'present']),
+        avi_api_update_method=dict(default='put',
+                                   choices=['put', 'patch']),
+        avi_api_patch_op=dict(choices=['add', 'replace', 'delete']),
         apdex_response_threshold=dict(type='int',),
         apdex_response_tolerated_factor=dict(type='float',),
         apdex_rtt_threshold=dict(type='int',),
@@ -473,6 +508,7 @@ def main():
         disable_se_analytics=dict(type='bool',),
         disable_server_analytics=dict(type='bool',),
         exclude_client_close_before_request_as_error=dict(type='bool',),
+        exclude_dns_policy_drop_as_significant=dict(type='bool',),
         exclude_gs_down_as_error=dict(type='bool',),
         exclude_http_error_codes=dict(type='list',),
         exclude_invalid_dns_domain_as_error=dict(type='bool',),
