@@ -204,7 +204,7 @@ import copy
 import re
 
 from ansible.module_utils.basic import AnsibleModule
-from ansible.module_utils.ce import ce_argument_spec, load_config, get_nc_config, set_nc_config
+from ansible.module_utils.network.cloudengine.ce import ce_argument_spec, load_config, get_nc_config, set_nc_config
 
 CE_NC_GET_NTP_AUTH_CONFIG = """
 <filter type="subtree">
