@@ -172,7 +172,7 @@ updates:
 
 from xml.etree import ElementTree
 from ansible.module_utils.basic import AnsibleModule
-from ansible.module_utils.ce import get_nc_config, set_nc_config, \
+from ansible.module_utils.network.cloudengine.ce import get_nc_config, set_nc_config, \
     ce_argument_spec, get_config, load_config, check_ip_addr
 
 # get snmp version
