@@ -119,34 +119,39 @@ commands:
 import re
 
 from ansible.module_utils.basic import AnsibleModule
-from ansible.module_utils.ios import get_config, load_config
-from ansible.module_utils.ios import ios_argument_spec, check_args
-from ansible.module_utils.network_common import ComplexList
+from ansible.module_utils.network.ios.ios import get_config, load_config
+from ansible.module_utils.network.ios.ios import ios_argument_spec, check_args
+from ansible.module_utils.network.common.utils import ComplexList
 
 _CONFIGURED_VRFS = None
 
+
 def has_vrf(module, vrf):
     global _CONFIGURED_VRFS
     if _CONFIGURED_VRFS is not None:
         return vrf in _CONFIGURED_VRFS
     config = get_config(module)
-    _CONFIGURED_VRFS = re.findall('vrf definition (\S+)', config)
+    _CONFIGURED_VRFS = re.findall(r'vrf definition (\S+)', config)
     return vrf in _CONFIGURED_VRFS
 
+
 def requires_vrf(module, vrf):
     if not has_vrf(module, vrf):
         module.fail_json(msg='vrf %s is not configured' % vrf)
 
+
 def diff_list(want, have):
     adds = [w for w in want if w not in have]
     removes = [h for h in have if h not in want]
     return (adds, removes)
 
+
 def map_obj_to_commands(want, have, module):
     commands = list()
     state = module.params['state']
 
-    needs_update = lambda x: want.get(x) is not None and (want.get(x) != have.get(x))
+    def needs_update(x):
+        return want.get(x) is not None and (want.get(x) != have.get(x))
 
     if state == 'absent':
         if have['hostname'] != 'Router':
@@ -226,7 +231,6 @@ def map_obj_to_commands(want, have, module):
                 else:
                     commands.append('ip domain list %s' % item['name'])
 
-
         if want['name_servers']:
             adds, removes = diff_list(want['name_servers'], have['name_servers'])
             for item in removes:
@@ -243,12 +247,14 @@ def map_obj_to_commands(want, have, module):
 
     return commands
 
+
 def parse_hostname(config):
-    match = re.search('^hostname (\S+)', config, re.M)
+    match = re.search(r'^hostname (\S+)', config, re.M)
     return match.group(1)
 
+
 def parse_domain_name(config):
-    match = re.findall('^ip domain name (?:vrf (\S+) )*(\S+)', config, re.M)
+    match = re.findall(r'^ip domain name (?:vrf (\S+) )*(\S+)', config, re.M)
     matches = list()
     for vrf, name in match:
         if not vrf:
@@ -256,8 +262,9 @@ def parse_domain_name(config):
         matches.append({'name': name, 'vrf': vrf})
     return matches
 
+
 def parse_domain_search(config):
-    match = re.findall('^ip domain list (?:vrf (\S+) )*(\S+)', config, re.M)
+    match = re.findall(r'^ip domain list (?:vrf (\S+) )*(\S+)', config, re.M)
     matches = list()
     for vrf, name in match:
         if not vrf:
@@ -265,8 +272,9 @@ def parse_domain_search(config):
         matches.append({'name': name, 'vrf': vrf})
     return matches
 
+
 def parse_name_servers(config):
-    match = re.findall('^ip name-server (?:vrf (\S+) )*(.*)', config, re.M)
+    match = re.findall(r'^ip name-server (?:vrf (\S+) )*(.*)', config, re.M)
     matches = list()
     for vrf, servers in match:
         if not vrf:
@@ -275,11 +283,13 @@ def parse_name_servers(config):
             matches.append({'server': server, 'vrf': vrf})
     return matches
 
+
 def parse_lookup_source(config):
-    match = re.search('ip domain lookup source-interface (\S+)', config, re.M)
+    match = re.search(r'ip domain lookup source-interface (\S+)', config, re.M)
     if match:
         return match.group(1)
 
+
 def map_config_to_obj(module):
     config = get_config(module)
     return {
@@ -291,6 +301,7 @@ def map_config_to_obj(module):
         'name_servers': parse_name_servers(config)
     }
 
+
 def map_params_to_obj(module):
     obj = {
         'hostname': module.params['hostname'],
@@ -324,6 +335,7 @@ def map_params_to_obj(module):
 
     return obj
 
+
 def main():
     """ Main entry point for Ansible module execution
     """
