@@ -24,7 +24,7 @@ description:
     between other tasks of the play.
   - Most keywords, loops and conditionals will only be applied to the imported tasks, not to this statement itself. If
     you want the opposite behavior, use M(include_role) instead. To better understand the difference you can read
-    U(https://docs.ansible.com/ansible/latest/playbooks_reuse_includes.html).
+    the L(Including and Importing Guide,../user_guide/playbooks_reuse_includes.html).
 version_added: "2.4"
 options:
   name:
